package validator

var iso4217 = map[string]struct{}{
	"AFN": {}, "EUR": {}, "ALL": {}, "DZD": {}, "USD": {},
	"AOA": {}, "XCD": {}, "ARS": {}, "AMD": {}, "AWG": {},
	"AUD": {}, "AZN": {}, "BSD": {}, "BHD": {}, "BDT": {},
	"BBD": {}, "BYN": {}, "BZD": {}, "XOF": {}, "BMD": {},
	"INR": {}, "BTN": {}, "BOB": {}, "BOV": {}, "BAM": {},
	"BWP": {}, "NOK": {}, "BRL": {}, "BND": {}, "BGN": {},
	"BIF": {}, "CVE": {}, "KHR": {}, "XAF": {}, "CAD": {},
	"KYD": {}, "CLP": {}, "CLF": {}, "CNY": {}, "COP": {},
	"COU": {}, "KMF": {}, "CDF": {}, "NZD": {}, "CRC": {},
	"HRK": {}, "CUP": {}, "CUC": {}, "ANG": {}, "CZK": {},
	"DKK": {}, "DJF": {}, "DOP": {}, "EGP": {}, "SVC": {},
	"ERN": {}, "SZL": {}, "ETB": {}, "FKP": {}, "FJD": {},
	"XPF": {}, "GMD": {}, "GEL": {}, "GHS": {}, "GIP": {},
	"GTQ": {}, "GBP": {}, "GNF": {}, "GYD": {}, "HTG": {},
	"HNL": {}, "HKD": {}, "HUF": {}, "ISK": {}, "IDR": {},
	"XDR": {}, "IRR": {}, "IQD": {}, "ILS": {}, "JMD": {},
	"JPY": {}, "JOD": {}, "KZT": {}, "KES": {}, "KPW": {},
	"KRW": {}, "KWD": {}, "KGS": {}, "LAK": {}, "LBP": {},
	"LSL": {}, "ZAR": {}, "LRD": {}, "LYD": {}, "CHF": {},
	"MOP": {}, "MKD": {}, "MGA": {}, "MWK": {}, "MYR": {},
	"MVR": {}, "MRU": {}, "MUR": {}, "XUA": {}, "MXN": {},
	"MXV": {}, "MDL": {}, "MNT": {}, "MAD": {}, "MZN": {},
	"MMK": {}, "NAD": {}, "NPR": {}, "NIO": {}, "NGN": {},
	"OMR": {}, "PKR": {}, "PAB": {}, "PGK": {}, "PYG": {},
	"PEN": {}, "PHP": {}, "PLN": {}, "QAR": {}, "RON": {},
	"RUB": {}, "RWF": {}, "SHP": {}, "WST": {}, "STN": {},
	"SAR": {}, "RSD": {}, "SCR": {}, "SLL": {}, "SGD": {},
	"XSU": {}, "SBD": {}, "SOS": {}, "SSP": {}, "LKR": {},
	"SDG": {}, "SRD": {}, "SEK": {}, "CHE": {}, "CHW": {},
	"SYP": {}, "TWD": {}, "TJS": {}, "TZS": {}, "THB": {},
	"TOP": {}, "TTD": {}, "TND": {}, "TRY": {}, "TMT": {},
	"UGX": {}, "UAH": {}, "AED": {}, "USN": {}, "UYU": {},
	"UYI": {}, "UYW": {}, "UZS": {}, "VUV": {}, "VES": {},
	"VND": {}, "YER": {}, "ZMW": {}, "ZWL": {}, "XBA": {},
	"XBB": {}, "XBC": {}, "XBD": {}, "XTS": {}, "XXX": {},
	"XAU": {}, "XPD": {}, "XPT": {}, "XAG": {},
}

var iso4217_numeric = map[int]struct{}{
	8: {}, 12: {}, 32: {}, 36: {}, 44: {},
	48: {}, 50: {}, 51: {}, 52: {}, 60: {},
	64: {}, 68: {}, 72: {}, 84: {}, 90: {},
	96: {}, 104: {}, 108: {}, 116: {}, 124: {},
	132: {}, 136: {}, 144: {}, 152: {}, 156: {},
	170: {}, 174: {}, 188: {}, 191: {}, 192: {},
	203: {}, 208: {}, 214: {}, 222: {}, 230: {},
	232: {}, 238: {}, 242: {}, 262: {}, 270: {},
	292: {}, 320: {}, 324: {}, 328: {}, 332: {},
	340: {}, 344: {}, 348: {}, 352: {}, 356: {},
	360: {}, 364: {}, 368: {}, 376: {}, 388: {},
	392: {}, 398: {}, 400: {}, 404: {}, 408: {},
	410: {}, 414: {}, 417: {}, 418: {}, 422: {},
	426: {}, 430: {}, 434: {}, 446: {}, 454: {},
	458: {}, 462: {}, 480: {}, 484: {}, 496: {},
	498: {}, 504: {}, 512: {}, 516: {}, 524: {},
	532: {}, 533: {}, 548: {}, 554: {}, 558: {},
	566: {}, 578: {}, 586: {}, 590: {}, 598: {},
	600: {}, 604: {}, 608: {}, 634: {}, 643: {},
	646: {}, 654: {}, 682: {}, 690: {}, 694: {},
	702: {}, 704: {}, 706: {}, 710: {}, 728: {},
	748: {}, 752: {}, 756: {}, 760: {}, 764: {},
	776: {}, 780: {}, 784: {}, 788: {}, 800: {},
	807: {}, 818: {}, 826: {}, 834: {}, 840: {},
	858: {}, 860: {}, 882: {}, 886: {}, 901: {},
	927: {}, 928: {}, 929: {}, 930: {}, 931: {},
	932: {}, 933: {}, 934: {}, 936: {}, 938: {},
	940: {}, 941: {}, 943: {}, 944: {}, 946: {},
	947: {}, 948: {}, 949: {}, 950: {}, 951: {},
	952: {}, 953: {}, 955: {}, 956: {}, 957: {},
	958: {}, 959: {}, 960: {}, 961: {}, 962: {},
	963: {}, 964: {}, 965: {}, 967: {}, 968: {},
	969: {}, 970: {}, 971: {}, 972: {}, 973: {},
	975: {}, 976: {}, 977: {}, 978: {}, 979: {},
	980: {}, 981: {}, 984: {}, 985: {}, 986: {},
	990: {}, 994: {}, 997: {}, 999: {},
}
